@@ -318,9 +318,8 @@ int pmt_cmtab_get(const char *spec, enum cmtab_field type, char **mountpoint,
 			break;
 
 		/* Guard against stale entries - verify that it is mounted. */
-		if (*crypto_device != NULL &&
-		    pmt_smtab_mounted(*crypto_device,
-		    *mountpoint, strcmp) > 0) {
+		if (*container != NULL &&
+		    pmt_smtab_mounted(*container, *mountpoint, strcmp) > 0) {
 			ret = 1;
 			break;
 		}
