@@ -12,6 +12,7 @@
 #include <sys/wait.h>
 #include <assert.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <signal.h>
 #include <stdbool.h>
@@ -20,8 +21,10 @@
 #include <string.h>
 #include <unistd.h>
 #include <libHX.h>
-#include <openssl/evp.h>
 #include "pam_mount.h"
+#ifdef HAVE_LIBCRYPTO
+#	include <openssl/evp.h>
+#endif
 
 /**
  * @object:		volume or mountpoint; used by remount
@@ -344,29 +347,30 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 
 static hxmc_t *mtcr_slurp_file(const char *file)
 {
-	hxmc_t *buf, *ln = NULL;
 	struct stat sb;
-	FILE *fp;
+	char tmp[4096];
+	hxmc_t *buf;
+	int fd;
 
-	fp = fopen(file, "r");
-	if (fp == NULL) {
+	if ((fd = open(file, O_RDONLY | O_BINARY)) < 0) {
 		fprintf(stderr, "Could not open %s: %s\n",
 		        file, strerror(errno));
 		return NULL;
 	}
 
-	if (fstat(fileno(fp), &sb) == 0 && S_ISREG(sb.st_mode))
+	if (fstat(fd, &sb) == 0 && S_ISREG(sb.st_mode))
 		buf = HXmc_meminit(NULL, sb.st_size);
 	else
 		buf = HXmc_meminit(NULL, 4096 / CHAR_BIT);
 
 	if (buf == NULL) {
-		fprintf(stderr, "%s\n", strerror(errno));
+		fprintf(stderr, "%s: %s\n", __func__, strerror(errno));
 	} else {
-		while (HX_getl(&ln, fp) != NULL)
-			HXmc_strcat(&buf, ln);
+		ssize_t ret;
+		while ((ret = read(fd, tmp, sizeof(tmp))) > 0)
+			HXmc_memcat(&buf, tmp, ret);
 	}
-	fclose(fp);
+	close(fd);
 	return buf;
 }
 
@@ -408,6 +412,7 @@ static int mtcr_mount(struct mount_options *opt)
 			return 0;
 		mount_request.trunc_keysize = HXmc_length(key);
 	} else {
+#ifdef HAVE_LIBCRYPTO
 		key = ehd_decrypt_key(opt->fsk_file, opt->fsk_hash,
 		      opt->fsk_cipher, opt->fsk_password);
 		if (key == NULL) {
@@ -415,6 +420,10 @@ static int mtcr_mount(struct mount_options *opt)
 			return 0;
 		}
 		mount_request.trunc_keysize = HXmc_length(key);
+#else
+		fprintf(stderr, "mtcrypt was compiled without OpenSSL support\n");
+		return 0;
+#endif
 	}
 
 	mount_request.key_data = key;
@@ -422,6 +431,8 @@ static int mtcr_mount(struct mount_options *opt)
 	if (opt->trunc_keysize != 0)
 		mount_request.trunc_keysize = opt->trunc_keysize;
 
+	w4rn("keysize=%u trunc_keysize=%u\n", mount_request.key_size,
+	     mount_request.trunc_keysize);
 	if ((ret = ehd_load(&mount_request, &mount_info)) < 0) {
 		fprintf(stderr, "ehd_load: %s\n", strerror(errno));
 		return 0;
@@ -660,8 +671,10 @@ int main(int argc, const char **argv)
 	pmtlog_prefix = HX_basename(*argv);
 
 	setenv("PATH", PMT_DFL_PATH, true);
+#ifdef HAVE_LIBCRYPTO
 	OpenSSL_add_all_ciphers();
 	OpenSSL_add_all_digests();
+#endif
 
 	/*
 	 * When invoking umount.crypt via the libtool helper script,
