@@ -1,7 +1,7 @@
 /*
  *	pam_mount
  *	Copyright (C) Elvis Pfützenreuter <epx@conectiva.com>, 2000
- *	Copyright © Jan Engelhardt, 2005 - 2009
+ *	Copyright © Jan Engelhardt, 2005 - 2010
  *	Copyright © Bastian Kleineidam, 2005
  *
  *	This file is part of pam_mount; you can redistribute it and/or
@@ -249,6 +249,10 @@ static int common_init(pam_handle_t *pamh, int argc, const char **argv)
 	pmtlog_path[PMTLOG_DBG][PMTLOG_SYSLOG] = Debug;
 	pmtlog_path[PMTLOG_DBG][PMTLOG_STDERR] = Debug;
 
+	ret = HX_init();
+	if (ret <= 0)
+		l0g("libHX init failed: %s\n", strerror(errno));
+
 	initconfig(&Config);
 	parse_pam_args(argc, argv);
 	/*
@@ -288,6 +292,53 @@ static int common_init(pam_handle_t *pamh, int argc, const char **argv)
 static void common_exit(void)
 {
 	pmt_sigpipe_setup(false);
+	HX_exit();
+}
+
+static void auth_grab_authtok(pam_handle_t *pamh, struct config *config)
+{
+	char *authtok = NULL;
+	int ret;
+
+	if (Args.get_pw_from_pam) {
+		char *ptr = NULL;
+
+		ret = pam_get_item(pamh, PAM_AUTHTOK, static_cast(const void **,
+		      static_cast(void *, &ptr)));
+		if (ret == PAM_SUCCESS && ptr != NULL)
+			authtok = xstrdup(ptr);
+	}
+	if (authtok == NULL && Args.get_pw_interactive) {
+		ret = read_password(pamh, config->msg_authpw, &authtok);
+		if (ret == PAM_SUCCESS && Args.propagate_pw) {
+			/*
+			 * pam_set_item() copies to PAM-internal memory.
+			 *
+			 * Using pam_set_item(PAM_AUTHTOK) here to make the
+			 * password that was just entered available to further
+			 * PAM modules.
+			 */
+			ret = pam_set_item(pamh, PAM_AUTHTOK, authtok);
+			if (ret != PAM_SUCCESS)
+				l0g("warning: failure to export password (%s)\n",
+				    pam_strerror(pamh, ret));
+		}
+	}
+
+	/*
+	 * Save auth token for pam_mount itself, since PAM_AUTHTOK
+	 * will be gone when the auth stage exits.
+	 */
+	if (authtok != NULL) {
+		ret = pam_set_data(pamh, "pam_mount_system_authtok", authtok,
+		                   clean_system_authtok);
+		if (ret == PAM_SUCCESS) {
+			if (mlock(authtok, strlen(authtok) + 1) < 0)
+				w4rn("mlock authtok: %s\n", strerror(errno));
+		} else {
+			l0g("error trying to save authtok for session code\n");
+		}
+	}
 }
 
 /**
@@ -305,48 +356,13 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_authenticate(pam_handle_t *pamh, int flags,
     int argc, const char **argv)
 {
 	int ret = PAM_SUCCESS;
-	char *authtok = NULL;
 
 	assert(pamh != NULL);
 
 	if ((ret = common_init(pamh, argc, argv)) != -1)
 		return ret;
 	w4rn(PACKAGE_STRING ": entering auth stage\n");
-
-	if (Args.get_pw_from_pam) {
-		char *ptr = NULL;
-
-		ret = pam_get_item(pamh, PAM_AUTHTOK, static_cast(const void **,
-		      static_cast(void *, &ptr)));
-		if (ret == PAM_SUCCESS && ptr != NULL)
-			authtok = xstrdup(ptr);
-	}
-	if (authtok == NULL && Args.get_pw_interactive) {
-		ret = read_password(pamh, Config.msg_authpw, &authtok);
-		if (ret == PAM_SUCCESS && Args.propagate_pw) {
-			/*
-			 * pam_set_item() copies to PAM-internal memory.
-			 *
-			 * Using pam_set_item(PAM_AUTHTOK) here to make the
-			 * password that was just entered available to further
-			 * PAM modules.
-			 */
-			ret = pam_set_item(pamh, PAM_AUTHTOK, authtok);
-			if (ret != PAM_SUCCESS)
-				l0g("warning: failure to export password (%s)\n",
-				    pam_strerror(pamh, ret));
-		}
-	}
-	if (authtok != NULL) {
-		ret = pam_set_data(pamh, "pam_mount_system_authtok", authtok,
-		                   clean_system_authtok);
-		if (ret == PAM_SUCCESS) {
-			if (mlock(authtok, strlen(authtok) + 1) < 0)
-				w4rn("mlock authtok: %s\n", strerror(errno));
-		} else {
-			l0g("error trying to save authtok for session code\n");
-		}
-	}
+	auth_grab_authtok(pamh, &Config);
 	common_exit();
 	/*
 	 * pam_mount is not really meant to be an auth module. So we should not
@@ -436,31 +452,44 @@ static int modify_pm_count(struct config *config, char *user,
 }
 
 /**
- * grab_authtok - get the password from PAM
+ * ses_grab_authtok - get the password from PAM
+ *
+ * Session stage: reretrieve password that the auth stage stored.
+ * If that does not work, use interactive prompting if enabled.
  */
-static char *grab_authtok(pam_handle_t *pamh)
+static char *ses_grab_authtok(pam_handle_t *pamh)
 {
-	char *system_authtok = NULL;
+	char *authtok = NULL;
 	int ret;
 
 	ret = pam_get_data(pamh, "pam_mount_system_authtok",
-	      static_cast(const void **, static_cast(void *, &system_authtok)));
-	if (ret != PAM_SUCCESS) {
-		if (Args.get_pw_interactive) {
-			ret = read_password(pamh, Config.msg_sessionpw,
-			      &system_authtok);
-			if (ret != PAM_SUCCESS)
-				l0g("warning: could not obtain password "
-				    "interactively either\n");
-		}
-		/*
-		 * Proceed without a password. Some volumes may not need one,
-		 * e.g. bind mounts and networked/unencrypted volumes.
-		 */
+	      static_cast(const void **, static_cast(void *, &authtok)));
+	if (ret == PAM_SUCCESS)
+		return authtok;
+
+	/* No stored password, get one, if allowed to. */
+	if (Args.get_pw_interactive) {
+		ret = read_password(pamh, Config.msg_sessionpw, &authtok);
+		if (ret != PAM_SUCCESS)
+			/* authtok is %NULL now */
+			l0g("warning: could not obtain password "
+			    "interactively either\n");
 	}
-	if (system_authtok == NULL)
-		system_authtok = xstrdup("");
-	return system_authtok;
+	if (authtok != NULL) {
+		ret = pam_set_data(pamh, "pam_mount_system_authtok",
+		      authtok, clean_system_authtok);
+		if (ret == PAM_SUCCESS) {
+			if (mlock(authtok, strlen(authtok) + 1) < 0)
+				w4rn("mlock authtok: %s\n", strerror(errno));
+		} else {
+			l0g("error trying to save authtok for session code\n");
+		}
+	}
+	/*
+	 * Always proceed, even if there is no password. Some volumes may not
+	 * need one, e.g. bind mounts and networked/unencrypted volumes.
+	 */
+	return authtok;
 }
 
 static int process_volumes(struct config *config, const char *authtok)
@@ -555,7 +584,7 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
 	}
 	if (Config.volume_list.items > 0)
 		/* There are some volumes, so grab a password. */
-		system_authtok = grab_authtok(pamh);
+		system_authtok = ses_grab_authtok(pamh);
 
 	misc_dump_id("Session open");
 	envpath_init(Config.path);
@@ -583,15 +612,15 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
 		w4rn("no volumes to mount\n");
 		ret = PAM_SUCCESS;
 	} else {
+		/*
+		 * If there are no global volumes, but luserconf volumes,
+		 * and we still have no password, ask for one now.
+		 */
 		if (system_authtok == NULL)
-			system_authtok = grab_authtok(pamh);
+			system_authtok = ses_grab_authtok(pamh);
 		ret = process_volumes(&Config, system_authtok);
 	}
 
-	if (system_authtok != NULL) {
-		memset(system_authtok, 0, strlen(system_authtok));
-		free(system_authtok);
-	}
 	modify_pm_count(&Config, Config.user, "1");
 	envpath_restore();
 	if (getuid() == 0)
@@ -648,11 +677,17 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
 PAM_EXTERN EXPORT_SYMBOL int pam_sm_close_session(pam_handle_t *pamh,
     int flags, int argc, const char **argv)
 {
-	int ret = PAM_SUCCESS;
 	const char *pam_user = NULL;
+	int ret;
 
 	assert(pamh != NULL);
 
+	ret = HX_init();
+	if (ret <= 0)
+		l0g("libHX init failed: %s\n", strerror(errno));
+	/* Up the refcount once again due to cleanconfig's delayed nature. */
+	HX_init();
+	ret = PAM_SUCCESS;
 	w4rn("received order to close things\n");
 	if (Config.volume_list.items == 0) {
 		w4rn("No volumes to umount\n");
@@ -678,6 +713,7 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_close_session(pam_handle_t *pamh,
 	if (chdir("/") != 0)
 		l0g("could not chdir\n");
 
+ out:
 	envpath_init(Config.path);
 	if (modify_pm_count(&Config, Config.user, "-1") > 0)
 		w4rn("%s seems to have other remaining open sessions\n",
@@ -686,11 +722,11 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_close_session(pam_handle_t *pamh,
 		umount_final(&Config);
 
 	envpath_restore();
- out:
 	/*
 	 * Note that PMConfig is automatically freed later in clean_config()
 	 */
 	w4rn("pam_mount execution complete\n");
+	HX_exit();
 	return ret;
 }
 
