@@ -58,7 +58,7 @@ struct callbackmap {
 
 struct pmt_command {
 	enum command_type type;
-	const char *fs, *def[9];
+	const char *fs, *def[11];
 	/*
 	 * You will need to enlarge @def whenever the compiler
 	 * (rightfully) complains.
@@ -578,7 +578,9 @@ static bool user_in_sgrp(const char *user, const char *grp, bool icase)
 	const char *const *wp;
 
 	if ((gent = getgrnam(grp)) == NULL) {
-		w4rn("getgrnam(\"%s\") failed: %s\n", grp, strerror(errno));
+		if (errno != 0)
+			w4rn("getgrnam(\"%s\") failed: %s\n",
+			     grp, strerror(errno));
 		return false;
 	}
 
@@ -601,7 +603,8 @@ static const char *rc_command(xmlNode *node, struct config *config,
 	char *arg, *wp;
 
 	if (config->level != CONTEXT_GLOBAL)
-		return "Tried to set command from user config\n";
+		return "Tried to set command from user config: "
+		       "not permitted\n";
 	if ((node = node->children) == NULL)
 		return NULL;
 
@@ -676,7 +679,8 @@ static const char *rc_luserconf(xmlNode *node, struct config *config,
 	char *s;
 
 	if (config->level != CONTEXT_GLOBAL)
-		return "Tried to set <luserconf> from user config";
+		return "Tried to set <luserconf> from user config: "
+		       "meaningless";
 	if ((pent = getpwnam(config->user)) == NULL)
 		return "Could not get password entry";
 	if ((s = xml_getprop(node, "name")) == NULL)
@@ -710,8 +714,8 @@ static const char *rc_mntoptions(xmlNode *node, struct config *config,
 	int ret;
 
 	if (config->level != CONTEXT_GLOBAL)
-		return "Tried to set <mntoptions allow=...> from user config";
-
+		return "Tried to set <mntoptions allow=...> from user config: "
+		       "not permitted";
 	if ((options = xml_getprop(node, "allow")) != NULL) {
 		if (!config->seen_mntoptions_allow) {
 			HXmap_free(config->options_allow);
@@ -1047,6 +1051,8 @@ static int rc_volume_cond_sgrp(const struct passwd *pwd, xmlNode *node)
 			continue;
 
 		if ((grp = getgrgid(pwd->pw_gid)) == NULL) {
+			if (errno == 0)
+				return 0;
 			w4rn("getgrgid(%ld) failed: %s\n",
 			     static_cast(long, pwd->pw_gid), strerror(errno));
 			return -1;
@@ -1055,9 +1061,8 @@ static int rc_volume_cond_sgrp(const struct passwd *pwd, xmlNode *node)
 		ret = rc_volume_cond_pgrp(pwd, parent);
 		if (ret < 0 || ret > 0)
 			return ret;
-		const char *nnn = signed_cast(const char *, node->content);
 		return user_in_sgrp(pwd->pw_name,
-		       (void*)nnn,
+		       signed_cast(const char *, node->content),
 		       parse_bool_f(xml_getprop(parent, "icase")));
 	}
 
@@ -1181,6 +1186,8 @@ static int rc_volume_cond(const char *user, xmlNode *node)
 	int ret;
 
 	if ((pwd_ent = getpwnam(user)) == NULL) {
+		if (errno == 0)
+			return 0;
 		l0g("getpwnam: %s\n", strerror(errno));
 		return -1;
 	}
@@ -1354,11 +1361,13 @@ static const struct pmt_command default_command[] = {
 	{CMD_NFSMOUNT,    "nfs",   {"mount", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(SERVER):%(VOLUME)", "%(MNTPT)", NULL}},
 	{CMD_NFSMOUNT,    "nfs4"},
 	{CMD_LCLMOUNT,    NULL,    {"mount", "-p0", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(VOLUME)", "%(MNTPT)", NULL}},
-	{CMD_CRYPTMOUNT,  "crypt", {"mount.crypt", "%(if %(CIPHER),-ocipher=%(CIPHER))", "%(if %(FSKEYCIPHER),-ofsk_cipher=%(FSKEYCIPHER))", "%(if %(FSKEYHASH),-ofsk_hash=%(FSKEYHASH))", "%(if %(FSKEYPATH),-okeyfile=%(FSKEYPATH))", "%(if %(OPTIONS),-o%(OPTIONS))", "%(VOLUME)", "%(MNTPT)", NULL}},
-	{CMD_CRYPTUMOUNT, "crypt", {"umount.crypt", "%(MNTPT)", NULL}},
+	{CMD_CRYPTMOUNT,  "crypt", {"mount", "-t", "crypt", "%(if %(CIPHER),-ocipher=%(CIPHER))", "%(if %(FSKEYCIPHER),-ofsk_cipher=%(FSKEYCIPHER))", "%(if %(FSKEYHASH),-ofsk_hash=%(FSKEYHASH))", "%(if %(FSKEYPATH),-okeyfile=%(FSKEYPATH))", "%(if %(OPTIONS),-o%(OPTIONS))", "%(VOLUME)", "%(MNTPT)", NULL}},
+	{CMD_CRYPTUMOUNT, "crypt", {"umount", "%(MNTPT)", NULL}},
 	{CMD_UMOUNT,     NULL,     {"umount", "%(MNTPT)", NULL}},
 	{CMD_FSCK,       NULL,     {"fsck", "-p", "%(FSCKTARGET)", NULL}},
 	{CMD_PMVARRUN,   NULL,     {"pmvarrun", "-u", "%(USER)", "-o", "%(OPERATION)", NULL}},
+	{CMD_FD0SSH,      NULL,    {"pmt-fd0ssh", NULL}},
+	{CMD_OFL,        NULL,     {"pmt-ofl", "-k%(SIGNAL)", "%(MNTPT)", NULL}},
 	{-1},
 };
 
@@ -1381,6 +1390,7 @@ static const struct callbackmap cf_tags[] = {
 	{"nfsmount",        rc_command,             CMD_NFSMOUNT},
 	{"ncpmount",        rc_command,             CMD_NCPMOUNT},
 	{"ncpumount",       rc_command,             CMD_NCPUMOUNT},
+	{"ofl",             rc_command,             CMD_OFL},
 	{"path",            rc_string,              CMDA_PATH},
 	{"pmvarrun",        rc_command,             CMD_PMVARRUN},
 	{"smbmount",        rc_command,             CMD_SMBMOUNT},
